import { planner, Plan, ProblemInfo, DomainInfo, parser } from 'pddl-workspace';
/** Abstract implementation of both sync/async planning service client. */
export declare abstract class PlannerService extends planner.Planner {
    constructor(plannerUrl: string, plannerConfiguration: planner.PlannerRunConfiguration, providerConfiguration: planner.ProviderConfiguration);
    abstract createRequestBody(domainFileInfo: DomainInfo, problemFileInfo: ProblemInfo): Promise<any>;
    abstract createUrl(): string;
    abstract processServerResponseBody(responseBody: any, planParser: parser.PddlPlannerOutputParser, parent: planner.PlannerResponseHandler, resolve: (plans: Plan[]) => void, reject: (error: Error) => void): void;
    plan(domainFileInfo: DomainInfo, problemFileInfo: ProblemInfo, planParser: parser.PddlPlannerOutputParser, parent: planner.PlannerResponseHandler): Promise<Plan[]>;
    /** Gets timeout in seconds. */
    abstract getTimeout(): number;
    parsePlanSteps(planSteps: any, planParser: parser.PddlPlannerOutputParser): void;
}
export interface HttpConnectionError {
    message: string;
    /** Host name or IP address e.g. 127.0.0.1 */
    address: string;
    code: string;
    errno: string;
    port: number;
}
export interface HttpConnectionRefusedError extends HttpConnectionError {
}
export declare function instanceOfHttpConnectionError(object: any): object is HttpConnectionError;
export declare function instanceOfHttpConnectionRefusedError(object: any): object is HttpConnectionRefusedError;
