/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.instanceOfHttpConnectionRefusedError = exports.instanceOfHttpConnectionError = exports.PlannerService = void 0;
const request = __importStar(require("request"));
const pddl_workspace_1 = require("pddl-workspace");
/** Abstract implementation of both sync/async planning service client. */
class PlannerService extends pddl_workspace_1.planner.Planner {
    constructor(plannerUrl, plannerConfiguration, providerConfiguration) {
        super(plannerUrl, plannerConfiguration, providerConfiguration);
    }
    plan(domainFileInfo, problemFileInfo, planParser, parent) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            parent.handleOutput(`Planning service: ${this.plannerPath}\nDomain: ${domainFileInfo.name}, Problem: ${problemFileInfo.name}\n`);
            // eslint-disable-next-line @typescript-eslint/no-explicit-any
            let requestHeader = {};
            if (((_a = this.plannerConfiguration.authentication) === null || _a === void 0 ? void 0 : _a.getToken()) !== undefined) {
                requestHeader = {
                    "Authorization": "Bearer " + this.plannerConfiguration.authentication.getToken()
                };
            }
            if (parent.providePlannerOptions({ domain: domainFileInfo, problem: problemFileInfo }).some(op => op.length)) {
                // todo: promote this to the planner factory
                console.warn("Search Debugger is not supported by planning services. Only planner executable may support it.");
            }
            const requestBody = yield this.createRequestBody(domainFileInfo, problemFileInfo);
            if (!requestBody) {
                return [];
            }
            const url = this.createUrl();
            const timeoutInSec = this.getTimeout();
            // eslint-disable-next-line @typescript-eslint/no-this-alias
            const that = this;
            return new Promise(function (resolve, reject) {
                request.post({ url: url, headers: requestHeader, body: requestBody, json: true, timeout: timeoutInSec * 1000 * 1.1 }, (err, httpResponse, responseBody) => {
                    if (err !== null) {
                        reject(err);
                        return;
                    }
                    if (that.plannerConfiguration.authentication) {
                        if (httpResponse) {
                            if (httpResponse.statusCode === 400) {
                                const message = "Authentication failed. Please login or update tokens.";
                                const error = new Error(message);
                                reject(error);
                                return;
                            }
                            else if (httpResponse.statusCode === 401) {
                                const message = "Invalid token. Please update tokens.";
                                const error = new Error(message);
                                reject(error);
                                return;
                            }
                        }
                    }
                    if (httpResponse && httpResponse.statusCode > 202) {
                        const notificationMessage = `PDDL Planning Service returned code ${httpResponse.statusCode} ${httpResponse.statusMessage}`;
                        const error = new Error(notificationMessage);
                        reject(error);
                        return;
                    }
                    that.processServerResponseBody(responseBody, planParser, parent, resolve, reject);
                });
            });
        });
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    parsePlanSteps(planSteps, planParser) {
        var _a;
        for (let index = 0; index < planSteps.length; index++) {
            const planStep = planSteps[index];
            const fullActionName = planStep["name"].replace('(', '').replace(')', '');
            const time = (_a = planStep["time"]) !== null && _a !== void 0 ? _a : (index + 1) * planParser.options.epsilon;
            let duration = planStep["duration"];
            const isDurative = duration !== undefined && duration !== null;
            duration = duration !== null && duration !== void 0 ? duration : planParser.options.epsilon;
            const planStepObj = new pddl_workspace_1.PlanStep(time, fullActionName, isDurative, duration, index);
            planParser.appendStep(planStepObj);
        }
        planParser.onPlanFinished();
    }
}
exports.PlannerService = PlannerService;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function instanceOfHttpConnectionError(object) {
    return 'address' in object && 'port' in object && 'code' in object && 'message' in object;
}
exports.instanceOfHttpConnectionError = instanceOfHttpConnectionError;
// eslint-disable-next-line @typescript-eslint/no-explicit-any
function instanceOfHttpConnectionRefusedError(object) {
    return (instanceOfHttpConnectionError(object)) && object.code === 'ECONNREFUSED';
}
exports.instanceOfHttpConnectionRefusedError = instanceOfHttpConnectionRefusedError;
//# sourceMappingURL=PlannerService.js.map