/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.PlannerSyncService = void 0;
const PlannerService_1 = require("./PlannerService");
/** Wraps the `/solve` planning web service interface. */
class PlannerSyncService extends PlannerService_1.PlannerService {
    constructor(plannerUrl, plannerConfiguration, providerConfiguration) {
        super(plannerUrl, plannerConfiguration, providerConfiguration);
    }
    createUrl() {
        let url = this.plannerPath;
        if (this.plannerConfiguration.options) {
            url = `${url}?${this.plannerConfiguration.options}`;
        }
        return url;
    }
    getTimeout() {
        return 60;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    createRequestBody(domainFileInfo, problemFileInfo) {
        const body = {
            "domain": domainFileInfo.getText(),
            "problem": problemFileInfo.getText()
        };
        return Promise.resolve(body);
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    processServerResponseBody(responseBody, planParser, callbacks, resolve, reject) {
        const status = responseBody["status"];
        if (status === "error") {
            const result = responseBody["result"];
            const resultOutput = result["output"];
            if (resultOutput) {
                callbacks.handleOutput(resultOutput);
            }
            const resultError = result["error"];
            if (resultError) {
                callbacks.handleOutput(resultError);
                resolve([]);
            }
            else {
                reject(new Error("An error occurred while solving the planning problem: " + JSON.stringify(result)));
            }
            return;
        }
        else if (status !== "ok") {
            reject(new Error(`Planner service failed with status ${status}.`));
            return;
        }
        const result = responseBody["result"];
        const resultOutput = result["output"];
        if (resultOutput) {
            callbacks.handleOutput(resultOutput);
        }
        this.parsePlanSteps(result['plan'], planParser);
        const plans = planParser.getPlans();
        if (plans.length > 0) {
            callbacks.handleOutput(plans[0].getText() + '\n');
        }
        else {
            callbacks.handleOutput('No plan found.');
        }
        resolve(plans);
    }
}
exports.PlannerSyncService = PlannerSyncService;
//# sourceMappingURL=PlannerSyncService.js.map