"use strict";
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
Object.defineProperty(exports, "__esModule", { value: true });
exports.checkResponseForError = exports.PlanningDomains = void 0;
const CatalogEntry_1 = require("./CatalogEntry");
const httpUtils_1 = require("../httpUtils");
/* eslint-disable @typescript-eslint/no-explicit-any */
class PlanningDomains {
    parseCollection(collectionJson) {
        return new CatalogEntry_1.Collection(collectionJson["collection_id"], collectionJson["collection_name"], collectionJson["description"], JSON.parse(collectionJson["domain_set"]));
    }
    async getCollections() {
        const url = PlanningDomains.URL + "collections";
        const jsonOutput = await getJsonResult(url);
        return jsonOutput
            .map((collectionJson) => this.parseCollection(collectionJson));
    }
    parseDomain(domainJson) {
        return new CatalogEntry_1.Domain(domainJson["domain_id"], domainJson["domain_name"], domainJson["description"]);
    }
    async getDomains(collection) {
        const url = PlanningDomains.URL + "domains/" + collection.id;
        const jsonOutput = await getJsonResult(url);
        return jsonOutput
            .map((domainJson) => this.parseDomain(domainJson))
            .sort(compareCatalogEntry);
    }
    parseProblem(problemJson) {
        return new CatalogEntry_1.Problem(problemJson["problem_id"], problemJson["problem"], problemJson["domain_url"], problemJson["problem_url"]);
    }
    async getProblems(domain) {
        const url = PlanningDomains.URL + "problems/" + domain.id;
        const jsonOutput = await getJsonResult(url);
        return jsonOutput
            .map((problemJson) => this.parseProblem(problemJson))
            .sort(compareCatalogEntry);
    }
}
exports.PlanningDomains = PlanningDomains;
PlanningDomains.AUTHORITY = "api.planning.domains";
PlanningDomains.URL = "https://api.planning.domains/json/classical/";
async function getJsonResult(url) {
    const response = await httpUtils_1.getJson(url);
    checkResponseForError(response);
    return response["result"];
}
function checkResponseForError(response) {
    if (response["error"]) {
        throw new Error(response["message"]);
    }
}
exports.checkResponseForError = checkResponseForError;
function compareCatalogEntry(a, b) {
    if (a.label && b.label && a.label < b.label) {
        return -1;
    }
    else if (a.label && b.label && a.label > b.label) {
        return 1;
    }
    else {
        return 0;
    }
}
//# sourceMappingURL=PlanningDomains.js.map