/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.Delegate = void 0;
const vscode_1 = require("vscode");
class Delegate {
    createOperator(label, detail, documentation, snippet) {
        return this.createCompletionItem(label, detail, documentation, vscode_1.CompletionItemKind.Operator, snippet);
    }
    createParameterized(label, detail, documentation, snippet) {
        return this.createCompletionItem(label, detail, documentation, vscode_1.CompletionItemKind.TypeParameter, snippet);
    }
    createCompletionItem(label, detail, documentation, kind, snippet) {
        const item = new vscode_1.CompletionItem(label, kind);
        item.detail = detail;
        item.documentation = documentation;
        if (snippet) {
            item.insertText = snippet;
        }
        return item;
    }
    static toNamesCsv(variables) {
        return variables
            .map(var1 => var1.name)
            .join(',');
    }
    static toTypeLessNamesCsv(variables) {
        return variables
            .map(var1 => var1.declaredNameWithoutTypes)
            .join(',');
    }
    getObjects(allTypeObjects, types) {
        return types.map(typeName => allTypeObjects.getTypeCaseInsensitive(typeName))
            .filter(typeObjects => !!typeObjects)
            .map(typeObjects => typeObjects.getObjects())
            .reduce((x, y) => x.concat(y), []); // flat map
    }
    getTypesInvolved(variables, domainFile) {
        const typesDirectlyInvolved = variables.map(p => p.parameters)
            .reduce((x, y) => x.concat(y), []) // flat map
            .map(p => p.type)
            .filter((v, i, a) => a.indexOf(v) === i); // distinct
        const typesInheriting = typesDirectlyInvolved
            .map(type1 => domainFile.getTypesInheritingFrom(type1))
            .reduce((x, y) => x.concat(y), []);
        return typesInheriting.concat(typesDirectlyInvolved);
    }
    getSymmetricPredicates(domainFile) {
        return domainFile.getPredicates().filter(this.isSymmetric);
    }
    getSymmetricFunctions(domainFile) {
        return domainFile.getFunctions().filter(this.isSymmetric);
    }
    isSymmetric(variable) {
        // the predicate has exactly 2 parameters
        return variable.parameters.length === 2
            // and the parameters are of the same type
            && variable.parameters[0].type === variable.parameters[1].type;
    }
}
exports.Delegate = Delegate;
//# sourceMappingURL=Delegate.js.map