/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.OperatorDelegate = void 0;
const vscode_1 = require("vscode");
const Delegate_1 = require("./Delegate");
const pddl_workspace_1 = require("pddl-workspace");
class OperatorDelegate extends Delegate_1.Delegate {
    constructor() {
        super();
    }
    getOperatorItems() {
        if (!this.operatorItems) {
            this.operatorItems = [
                this.createOperator('and', 'Logical conjunction', new vscode_1.MarkdownString('Example:').appendCodeblock('(and (fact1)(fact2))', pddl_workspace_1.PDDL)),
                this.createOperator('not', 'Logical negation', new vscode_1.MarkdownString('Example:').appendCodeblock('(not (fact1))', pddl_workspace_1.PDDL)),
                this.createOperator('or', 'Logical disjunction', new vscode_1.MarkdownString('Example:').appendCodeblock('(or (fact1) (fact2))', pddl_workspace_1.PDDL)),
                this.createOperator('=', 'Equality', 'Evaluates whether two numeric values are equal.'),
                this.createOperator('>', 'Greater than', ''),
                this.createOperator('<', 'Less than', ''),
                this.createOperator('>=', 'Greater than or equal', ''),
                this.createOperator('<=', 'Less than or equal', ''),
                this.createOperator('+', 'Numerical addition', new vscode_1.MarkdownString('Example:').appendCodeblock('(+ (function1) 1.0)', pddl_workspace_1.PDDL)),
                this.createOperator('-', 'Numerical subtraction', new vscode_1.MarkdownString('Example:').appendCodeblock('(- (function1) 1.0)', pddl_workspace_1.PDDL)),
                this.createOperator('/', 'Numerical division', new vscode_1.MarkdownString('Example:').appendCodeblock('(/ (function1) 2)', pddl_workspace_1.PDDL)),
                this.createOperator('*', 'Numerical multiplication', new vscode_1.MarkdownString('Example:').appendCodeblock('(* (function1) 2)', pddl_workspace_1.PDDL)),
                this.createParameterized('forall', 'For all condition', new vscode_1.MarkdownString('Conjunction condition for all objects of specified type. For example:').appendCodeblock('(forall (?p - product) (available ?p))', pddl_workspace_1.PDDL), new vscode_1.SnippetString('forall ($1) $0')),
                this.createParameterized('exists', 'Existential condition', new vscode_1.MarkdownString('Condition such as').appendCodeblock('(exists (?p - product)(available ?p))', pddl_workspace_1.PDDL), new vscode_1.SnippetString('exists ($1) $0')),
            ];
        }
        return this.operatorItems;
    }
}
exports.OperatorDelegate = OperatorDelegate;
//# sourceMappingURL=OperatorDelegate.js.map