/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.TypeDelegate = void 0;
const vscode_1 = require("vscode");
const Delegate_1 = require("./Delegate");
class TypeDelegate extends Delegate_1.Delegate {
    constructor(workspace) {
        super();
        this.workspace = workspace;
    }
    getTypeItems(fileInfo) {
        return this.getTypes(fileInfo).map(t => this.createType(t));
    }
    getTypes(fileInfo) {
        const domainInfo = this.workspace.asDomain(fileInfo);
        return domainInfo ? domainInfo.getTypes() : [];
    }
    createType(typeName) {
        const completionItem = this.createCompletionItem(typeName, 'Type', '', vscode_1.CompletionItemKind.Class);
        completionItem.insertText = ' ' + typeName; // prefix with a space for formatting
        completionItem.filterText = '- ' + typeName;
        return completionItem;
    }
}
exports.TypeDelegate = TypeDelegate;
//# sourceMappingURL=TypeDelegate.js.map