/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.VariableDelegate = void 0;
const vscode_1 = require("vscode");
const Delegate_1 = require("./Delegate");
const SymbolUtils_1 = require("../symbols/SymbolUtils");
class VariableDelegate extends Delegate_1.Delegate {
    constructor(workspace) {
        super();
        this.workspace = workspace;
        this.symbolUtils = new SymbolUtils_1.SymbolUtils(workspace);
    }
    getVariableItems(fileInfo) {
        const domainInfo = this.workspace.pddlWorkspace.asDomain(fileInfo);
        if (!domainInfo) {
            return [];
        }
        const predicates = domainInfo.getPredicates().map(p => this.createPredicate(p));
        const functions = domainInfo.getFunctions().map(f => this.createFunction(f));
        const derived = domainInfo.getDerived().map(d => this.createDerived(d));
        return predicates.concat(functions).concat(derived);
    }
    createPredicate(predicate) {
        return this.createSymbol(predicate, 'Predicate', vscode_1.CompletionItemKind.Value);
    }
    createFunction(functionSymbol) {
        return this.createSymbol(functionSymbol, 'Function', vscode_1.CompletionItemKind.Unit);
    }
    createDerived(derivedSymbol) {
        return this.createSymbol(derivedSymbol, 'Derived predicate/function', vscode_1.CompletionItemKind.Interface);
    }
    createSymbol(symbol, title, kind) {
        const markdownString = this.symbolUtils.createSymbolMarkdownDocumentation(undefined, `(${symbol.declaredName})`, symbol.getDocumentation());
        const completionItem = this.createCompletionItem(symbol.declaredName, title, markdownString, kind);
        completionItem.insertText = symbol.declaredNameWithoutTypes;
        return completionItem;
    }
}
exports.VariableDelegate = VariableDelegate;
//# sourceMappingURL=VariableDelegate.js.map