/* --------------------------------------------------------------------------------------------
* Copyright (c) Jan Dolejsi. All rights reserved.
* Licensed under the MIT License. See License.txt in the project root for license information.
* ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.DomainDiagnostics = exports.UNUSED = void 0;
const pddl_workspace_1 = require("pddl-workspace");
const vscode_1 = require("vscode");
const pddl_workspace_2 = require("pddl-workspace");
const utils_1 = require("../utils");
exports.UNUSED = 'unused';
/**
 * Domain file diagnostics.
 */
class DomainDiagnostics {
    constructor(codePddlWorkspace) {
        this.diagnosticCollection = vscode_1.languages.createDiagnosticCollection(pddl_workspace_2.PDDL + '2');
        codePddlWorkspace.pddlWorkspace.on(pddl_workspace_1.PddlWorkspace.UPDATED, (fileInfo) => {
            if (fileInfo.isDomain()) {
                this.validateDomain(fileInfo);
            }
        });
    }
    validateDomain(domainInfo) {
        const predicateDiagnostic = domainInfo.getPredicates()
            .map(p => this.toUnusedDiagnostic(domainInfo, p, 'predicate'))
            .filter(diagnostic => !!diagnostic)
            .map(diagnostics => diagnostics);
        const functionDiagnostic = domainInfo.getFunctions()
            .map(p => this.toUnusedDiagnostic(domainInfo, p, 'function'))
            .filter(diagnostic => !!diagnostic)
            .map(diagnostics => diagnostics);
        const diagnostics = predicateDiagnostic.concat(functionDiagnostic);
        this.diagnosticCollection.set(utils_1.toUri(domainInfo.fileUri), diagnostics);
    }
    toUnusedDiagnostic(domainInfo, variable, variableType) {
        const references = domainInfo.getVariableReferences(variable);
        if (references.length === 1) {
            const diagnostic = new vscode_1.Diagnostic(utils_1.toRange(references[0]), `Unused ${variableType} (${variable.declaredName})`, vscode_1.DiagnosticSeverity.Hint);
            diagnostic.tags = [vscode_1.DiagnosticTag.Unnecessary];
            diagnostic.code = exports.UNUSED;
            return diagnostic;
        }
        return undefined;
    }
}
exports.DomainDiagnostics = DomainDiagnostics;
//# sourceMappingURL=DomainDiagnostics.js.map