/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.DocumentInsetCodeLens = exports.DocumentCodeLens = exports.WebviewPanelAdapter = void 0;
const vscode_1 = require("vscode");
class WebviewPanelAdapter {
    constructor(panel) {
        this.panel = panel;
    }
    canReveal() {
        return true;
    }
    dispose() {
        this.panel.dispose();
    }
    reveal(displayColumn) {
        this.panel.reveal(displayColumn);
    }
    isVisible() {
        return this.panel.visible;
    }
    get html() {
        return this.panel.webview.html;
    }
    set html(value) {
        this.panel.webview.html = value;
    }
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    postMessage(message) {
        return this.panel.webview.postMessage(message);
    }
    get isInset() {
        return false;
    }
    get webview() {
        return this.panel.webview;
    }
}
exports.WebviewPanelAdapter = WebviewPanelAdapter;
class DocumentCodeLens extends vscode_1.CodeLens {
    constructor(document, range, command) {
        super(range, command);
        this.document = document;
    }
    getDocument() {
        return this.document;
    }
}
exports.DocumentCodeLens = DocumentCodeLens;
class DocumentInsetCodeLens extends DocumentCodeLens {
    constructor(document, range, line, command) {
        super(document, range, command);
        this.line = line;
    }
    getLine() {
        return this.line;
    }
}
exports.DocumentInsetCodeLens = DocumentInsetCodeLens;
//# sourceMappingURL=view.js.map