/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi 2021. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.handleValStepError = void 0;
const ai_planning_val_1 = require("ai-planning-val");
const vscode_1 = require("vscode");
const path = require("path");
// eslint-disable-next-line @typescript-eslint/no-explicit-any
async function handleValStepError(err, valStepPath) {
    var _a;
    if (err instanceof ai_planning_val_1.ValStepError) {
        try {
            const exportCase = "Export valstep case...";
            const choice = await vscode_1.window.showErrorMessage("ValStep failed to evaluate the plan values.", exportCase, "Ignore");
            if (choice === exportCase) {
                const targetPathUris = await vscode_1.window.showOpenDialog({
                    canSelectFolders: true, canSelectFiles: false,
                    defaultUri: vscode_1.Uri.file(path.dirname(err.domain.fileUri.fsPath)),
                    openLabel: 'Select target folder'
                });
                if (!targetPathUris) {
                    return;
                }
                const targetPath = targetPathUris[0];
                const outputPath = await ai_planning_val_1.ValStep.storeError(err, targetPath.fsPath, valStepPath);
                const success = await vscode_1.env.openExternal(vscode_1.Uri.file(outputPath));
                if (!success) {
                    vscode_1.window.showErrorMessage(`Files for valstep bug report: ${outputPath}.`);
                }
            }
        }
        catch (err1) {
            console.log(err1);
        }
    }
    else {
        vscode_1.window.showWarningMessage((_a = err === null || err === void 0 ? void 0 : err.message) !== null && _a !== void 0 ? _a : err);
    }
}
exports.handleValStepError = handleValStepError;
//# sourceMappingURL=valStepErrorHandler.js.map