/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.TestsManifest = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const fs_1 = require("fs");
const Test_1 = require("./Test");
const vscode_1 = require("vscode");
/**
 * Tests manifest
 */
class TestsManifest {
    constructor(defaultDomain, defaultProblem, defaultOptions, uri) {
        this.defaultDomain = defaultDomain;
        this.defaultProblem = defaultProblem;
        this.defaultOptions = defaultOptions;
        this.uri = uri;
        this.testCases = [];
        this.path = uri.fsPath;
    }
    static fromJSON(path, json, context) {
        const defaultDomain = json["defaultDomain"];
        const defaultProblem = json["defaultProblem"];
        const defaultOptions = json["defaultOptions"];
        const uri = vscode_1.Uri.file(path);
        const manifest = new TestsManifest(defaultDomain, defaultProblem, defaultOptions, uri);
        const tests = json["cases"] ? json["cases"].map((t) => Test_1.Test.fromJSON(t, context)) : [];
        tests.forEach(case1 => manifest.addCase(case1));
        return manifest;
    }
    addCase(testCase) {
        testCase.setManifest(this);
        this.testCases.push(testCase);
    }
    static load(path, context) {
        // todo: use workspace.fs.readFile, but deal with the async nature
        const settings = fs_1.readFileSync(path);
        const json = JSON.parse(settings.toLocaleString());
        return TestsManifest.fromJSON(path, json, context);
    }
    async store() {
        const obj = {};
        if (this.defaultDomain !== undefined) {
            obj["defaultDomain"] = this.defaultDomain;
        }
        if (this.defaultProblem !== undefined) {
            obj["defaultProblem"] = this.defaultProblem;
        }
        if (this.defaultOptions !== undefined) {
            obj["defaultOptions"] = this.defaultOptions;
        }
        const cases = [];
        this.testCases.forEach(test => cases.push(test.toJSON()));
        if (cases.length > 0) {
            obj["cases"] = cases;
        }
        const json = JSON.stringify(obj, null, 2);
        try {
            await vscode_1.workspace.fs.writeFile(this.uri, Buffer.from(json, 'utf8'));
        }
        catch (err) {
            vscode_1.window.showErrorMessage(`Error saving test case manifest ${err.name}: ${err.message}`);
        }
    }
}
exports.TestsManifest = TestsManifest;
//# sourceMappingURL=TestsManifest.js.map