"use strict";
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
Object.defineProperty(exports, "__esModule", { value: true });
exports.SessionDocumentContentProvider = void 0;
const vscode_1 = require("vscode");
const SessionRepository_1 = require("./SessionRepository");
const path_1 = require("path");
/**
 * Provides the content of the Planning.Domains session documents as fetched from the server i.e. without the local edits.
 * This is used for the source control diff.
 */
class SessionDocumentContentProvider {
    constructor() {
        this._onDidChange = new vscode_1.EventEmitter();
        // map of workspace folders (key is Uri::toString()) and the checked-out sessions content
        this.sessions = new Map();
    }
    get onDidChange() {
        return this._onDidChange.event;
    }
    dispose() {
        this._onDidChange.dispose();
    }
    updated(folder, newSession) {
        this.sessions.set(folder.uri.toString(), newSession);
        // let's assume all documents actually changed and notify the quick-diff
        newSession.files.forEach((_, fileName) => {
            const uri = SessionRepository_1.SessionRepository.createDocumentUri(folder.uri, fileName);
            this._onDidChange.fire(uri);
        });
    }
    provideTextDocumentContent(uri, token) {
        if (token.isCancellationRequested) {
            return "Canceled";
        }
        const folderUri = vscode_1.Uri.file(uri.path);
        // if the file is in a sub-folder, it is not member of the session
        if (uri.query.includes('/')) {
            return undefined;
        }
        const fileName = path_1.basename(uri.query);
        const session = this.sessions.get(folderUri.toString());
        if (!session) {
            return "Resource not found: " + uri.toString();
        }
        return session.files.get(fileName);
    }
}
exports.SessionDocumentContentProvider = SessionDocumentContentProvider;
//# sourceMappingURL=SessionDocumentContentProvider.js.map