/* --------------------------------------------------------------------------------------------
 * Copyright (c) Jan Dolejsi. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
'use strict';
Object.defineProperty(exports, "__esModule", { value: true });
exports.isFile = exports.isNotDirectory = exports.isDirectory = exports.doesNotExist = exports.exists = void 0;
const vscode_1 = require("vscode");
async function exists(uri) {
    try {
        await vscode_1.workspace.fs.stat(uri);
    }
    catch (err) {
        if (err instanceof vscode_1.FileSystemError && err.code === 'FileNotFound') {
            return false;
        }
        else {
            throw err;
        }
    }
    return true;
}
exports.exists = exists;
async function doesNotExist(uri) {
    return !(await exists(uri));
}
exports.doesNotExist = doesNotExist;
async function isDirectory(uri) {
    const stat = await vscode_1.workspace.fs.stat(uri);
    return stat.type === vscode_1.FileType.Directory;
}
exports.isDirectory = isDirectory;
async function isNotDirectory(uri) {
    return !(await isDirectory(uri));
}
exports.isNotDirectory = isNotDirectory;
async function isFile(uri) {
    const stat = await vscode_1.workspace.fs.stat(uri);
    return stat.type === vscode_1.FileType.File;
}
exports.isFile = isFile;
//# sourceMappingURL=workspaceFs.js.map